@@ -42,7 +42,7 @@ var hasCachedContent = false;
 
 var onMessage = function(msg) {
     switch ( msg.what ) {
-    case 'allFilterListsReloaded':
+    case 'staticFilteringDataChanged':
         renderFilterLists();
         break;
 
