@@ -19,6 +19,8 @@
     Home: https://github.com/gorhill/uBlock
 */
 
+/* global createSet */
+
 'use strict';
 
 /*******************************************************************************
@@ -78,8 +80,6 @@ if ( typeof vAPI !== 'object' ) {
 }
 vAPI.lock();
 
-vAPI.executionCost.start();
-
 vAPI.matchesProp = (function() {
     var docElem = document.documentElement;
     if ( typeof docElem.matches !== 'function' ) {
@@ -96,56 +96,36 @@ vAPI.matchesProp = (function() {
 /******************************************************************************/
 /******************************************************************************/
 
+// https://github.com/gorhill/uBlock/issues/2147
+
+vAPI.SafeAnimationFrame = function(callback) {
+    this.fid = this.tid = null;
+    this.callback = callback;
+};
+
+vAPI.SafeAnimationFrame.prototype.start = function() {
+    if ( this.fid !== null ) { return; }
+    this.fid = requestAnimationFrame(this.callback);
+    this.tid = vAPI.setTimeout(this.callback, 1200000);
+};
+
+vAPI.SafeAnimationFrame.prototype.clear = function() {
+    if ( this.fid === null ) { return; }
+    cancelAnimationFrame(this.fid);
+    clearTimeout(this.tid);
+    this.fid = this.tid = null;
+};
+
+/******************************************************************************/
+/******************************************************************************/
+/******************************************************************************/
+
 // The DOM filterer is the heart of uBO's cosmetic filtering.
 
 vAPI.domFilterer = (function() {
 
 /******************************************************************************/
 
-if ( typeof self.Set !== 'function' ) {
-    self.Set = function() {
-        this._set = [];
-        this._i = 0;
-        this.value = undefined;
-    };
-    self.Set.prototype = {
-        polyfill: true,
-        clear: function() {
-            this._set = [];
-        },
-        add: function(k) {
-            if ( this._set.indexOf(k) === -1 ) {
-                this._set.push(k);
-            }
-        },
-        delete: function(k) {
-            var pos = this._set.indexOf(k);
-            if ( pos !== -1 ) {
-                this._set.splice(pos, 1);
-                return true;
-            }
-            return false;
-        },
-        has: function(k) {
-            return this._set.indexOf(k) !== -1;
-        },
-        values: function() {
-            this._i = 0;
-            return this;
-        },
-        next: function() {
-            this.value = this._set[this._i];
-            this._i += 1;
-            return this;
-        }
-    };
-    Object.defineProperty(self.Set.prototype, 'size', {
-        get: function() { return this._set.length; }
-    });
-}
-
-/******************************************************************************/
-
 var shadowId = document.documentElement.shadowRoot !== undefined ?
     vAPI.randomToken():
     undefined;
@@ -157,11 +137,10 @@ var jobQueue = [
     { t: 'css-csel',  _0: [] }  // to manually hide (not incremental)
 ];
 
-var reParserEx = /:(?:matches-css|has|style|xpath)\(.+?\)$/;
+var reParserEx = /:(?:has|matches-css|matches-css-before|matches-css-after|style|xpath)\(.+?\)$/;
 
-var allExceptions = Object.create(null),
-    allSelectors = Object.create(null),
-    commitTimer = null,
+var allExceptions = createSet(),
+    allSelectors = createSet(),
     stagedNodes = [],
     matchesProp = vAPI.matchesProp,
     userCSS = vAPI.userCSS;
@@ -170,7 +149,7 @@ var allExceptions = Object.create(null),
 // Set() is used to implement this functionality.
 
 var complexSelectorsOldResultSet,
-    complexSelectorsCurrentResultSet = new Set();
+    complexSelectorsCurrentResultSet = createSet('object');
 
 /******************************************************************************/
 
@@ -222,48 +201,45 @@ var runHasJob = function(job, fn) {
     }
 };
 
-var csspropDictFromString = function(s) {
-    var aa = s.split(/;\s+|;$/),
-        i = aa.length,
-        dict = Object.create(null),
-        prop, pos;
-    while ( i-- ) {
-        prop = aa[i].trim();
-        if ( prop === '' ) { continue; }
-        pos = prop.indexOf(':');
-        if ( pos === -1 ) { continue; }
-        dict[prop.slice(0, pos).trim()] = prop.slice(pos + 1).trim();
+// '/' = ascii 0x2F */
+
+var parseMatchesCSSJob = function(raw) {
+    var prop = raw.trim();
+    if ( prop === '' ) { return null; }
+    var pos = prop.indexOf(':'),
+        v = pos !== -1 ? prop.slice(pos + 1).trim() : '',
+        vlen = v.length;
+    if (
+        vlen > 1 &&
+        v.charCodeAt(0) === 0x2F &&
+        v.charCodeAt(vlen-1) === 0x2F
+    ) {
+        try { v = new RegExp(v.slice(1, -1)); } catch(ex) { return null; }
     }
-    return dict;
+    return { k: prop.slice(0, pos).trim(), v: v };
 };
 
 var runMatchesCSSJob = function(job, fn) {
-    if ( job._2 === undefined ) {
-        if ( job._0.indexOf(':after', job._0.length - 6) !== -1 ) {
-            job._0 = job._0.slice(0, -6);
-            job._2 = ':after';
-        } else {
-            job._2 = null;
-        }
-    }
     var nodes = document.querySelectorAll(job._0),
         i = nodes.length;
     if ( i === 0 ) { return; }
     if ( typeof job._1 === 'string' ) {
-        job._1 = csspropDictFromString(job._1);
+        job._1 = parseMatchesCSSJob(job._1);
     }
-    var node, match, style;
+    if ( job._1 === null ) { return; }
+    var k = job._1.k,
+        v = job._1.v,
+        node, style, match;
     while ( i-- ) {
         node = nodes[i];
         style = window.getComputedStyle(node, job._2);
-        match = undefined;
-        for ( var prop in job._1 ) {
-            match = style[prop] === job._1[prop];
-            if ( match === false ) {
-                break;
-            }
+        if ( style === null ) { continue; } /* FF */
+        if ( v instanceof RegExp ) {
+            match = v.test(style[k]);
+        } else {
+            match = style[k] === v;
         }
-        if ( match === true ) {
+        if ( match ) {
             fn(node, job);
         }
     }
@@ -292,10 +268,13 @@ var runXpathJob = function(job, fn) {
 var domFilterer = {
     addedNodesHandlerMissCount: 0,
     removedNodesHandlerMissCount: 0,
+    commitTimer: null,
     disabledId: vAPI.randomToken(),
     enabled: true,
+    excludeId: undefined,
     hiddenId: vAPI.randomToken(),
     hiddenNodeCount: 0,
+    hiddenNodeEnforcer: false,
     loggerEnabled: undefined,
     styleTags: [],
 
@@ -308,7 +287,7 @@ var domFilterer = {
 
     addExceptions: function(aa) {
         for ( var i = 0, n = aa.length; i < n; i++ ) {
-            allExceptions[aa[i]] = true;
+            allExceptions.add(aa[i]);
         }
     },
 
@@ -324,10 +303,10 @@ var domFilterer = {
     //     xpath/hide
 
     addSelector: function(s) {
-        if ( allSelectors[s] || allExceptions[s] ) {
+        if ( allSelectors.has(s) || allExceptions.has(s) ) {
             return;
         }
-        allSelectors[s] = true;
+        allSelectors.add(s);
         var sel0 = s, sel1 = '';
         if ( s.charCodeAt(s.length - 1) === 0x29 ) {
             var parts = reParserEx.exec(s);
@@ -350,7 +329,13 @@ var domFilterer = {
         if ( sel1.lastIndexOf(':has', 0) === 0 ) {
             this.jobQueue.push({ t: 'has-hide', raw: s, _0: sel0, _1: sel1.slice(5, -1) });
         } else if ( sel1.lastIndexOf(':matches-css', 0) === 0 ) {
-            this.jobQueue.push({ t: 'matches-css-hide', raw: s, _0: sel0, _1: sel1.slice(13, -1) });
+            if ( sel1.lastIndexOf(':matches-css-before', 0) === 0 ) {
+                this.jobQueue.push({ t: 'matches-css-hide', raw: s, _0: sel0, _1: sel1.slice(20, -1), _2: ':before' });
+            } else if ( sel1.lastIndexOf(':matches-css-after', 0) === 0 ) {
+                this.jobQueue.push({ t: 'matches-css-hide', raw: s, _0: sel0, _1: sel1.slice(19, -1), _2: ':after' });
+            } else {
+                this.jobQueue.push({ t: 'matches-css-hide', raw: s, _0: sel0, _1: sel1.slice(13, -1), _2: null });
+            }
         } else if ( sel1.lastIndexOf(':style', 0) === 0 ) {
             this.job1._0.push(sel0 + ' { ' + sel1.slice(7, -1) + ' }');
             this.job1._1 = undefined;
@@ -366,6 +351,19 @@ var domFilterer = {
         }
     },
 
+    addStyleTag: function(text) {
+        var styleTag = document.createElement('style');
+        styleTag.setAttribute('type', 'text/css');
+        styleTag.textContent = text;
+        if ( document.head ) {
+            document.head.appendChild(styleTag);
+        }
+        this.styleTags.push(styleTag);
+        if ( userCSS ) {
+            userCSS.add(text);
+        }
+    },
+
     checkStyleTags_: function() {
         var doc = document,
             html = doc.documentElement,
@@ -402,9 +400,7 @@ var domFilterer = {
     },
 
     commit_: function() {
-        vAPI.executionCost.start();
-
-        commitTimer = null;
+        this.commitTimer.clear();
 
         var beforeHiddenNodeCount = this.hiddenNodeCount,
             styleText = '', i, n;
@@ -421,19 +417,6 @@ var domFilterer = {
             this.job1._0.length = 0;
         }
 
-        if ( styleText !== '' ) {
-            var styleTag = document.createElement('style');
-            styleTag.setAttribute('type', 'text/css');
-            styleTag.textContent = styleText;
-            if ( document.head ) {
-                document.head.appendChild(styleTag);
-            }
-            this.styleTags.push(styleTag);
-            if ( userCSS ) {
-                userCSS.add(styleText);
-            }
-        }
-
         // Simple selectors: incremental.
 
         // Stock job 2 = simple css selectors/hide
@@ -447,7 +430,7 @@ var domFilterer = {
 
         // Complex selectors: non-incremental.
         complexSelectorsOldResultSet = complexSelectorsCurrentResultSet;
-        complexSelectorsCurrentResultSet = new Set();
+        complexSelectorsCurrentResultSet = createSet('object');
 
         // Stock job 3 = complex css selectors/hide
         // The handling of these can be considered optional, since they are
@@ -462,13 +445,25 @@ var domFilterer = {
             this.runJob(this.jobQueue[i], complexHideNode);
         }
 
+        // https://github.com/gorhill/uBlock/issues/1912
+        //   If one or more nodes have been manually hidden, insert a style tag
+        //   targeting these manually hidden nodes. For browsers supporting
+        //   user styles, this allows uBO to win.
         var commitHit = this.hiddenNodeCount !== beforeHiddenNodeCount;
         if ( commitHit ) {
+            if ( this.hiddenNodeEnforcer === false ) {
+                styleText += '\n:root *[' + this.hiddenId + '][hidden] { display: none !important; }';
+                this.hiddenNodeEnforcer = true;
+            }
             this.addedNodesHandlerMissCount = 0;
         } else {
             this.addedNodesHandlerMissCount += 1;
         }
 
+        if ( styleText !== '' ) {
+            this.addStyleTag(styleText);
+        }
+
         // Un-hide nodes previously hidden.
         i = complexSelectorsOldResultSet.size;
         if ( i !== 0 ) {
@@ -490,8 +485,6 @@ var domFilterer = {
                 503
             );
         }
-
-        vAPI.executionCost.stop('domFilterer.commit_');
     },
 
     commit: function(nodes, commitNow) {
@@ -501,21 +494,23 @@ var domFilterer = {
             stagedNodes = stagedNodes.concat(nodes);
         }
         if ( commitNow ) {
-            if ( commitTimer !== null ) {
-                window.cancelAnimationFrame(commitTimer);
-            }
+            this.commitTimer.clear();
             this.commit_();
             return;
         }
-        if ( commitTimer === null ) {
-            commitTimer = window.requestAnimationFrame(this.commit_.bind(this));
+        this.commitTimer.start();
+    },
+
+    getExcludeId: function() {
+        if ( this.excludeId === undefined ) {
+            this.excludeId = vAPI.randomToken();
         }
+        return this.excludeId;
     },
 
     hideNode: function(node) {
-        if ( node[this.hiddenId] !== undefined ) {
-            return;
-        }
+        if ( node[this.hiddenId] !== undefined ) { return; }
+        if ( this.excludeId !== undefined && node[this.excludeId] ) { return; }
         node.setAttribute(this.hiddenId, '');
         this.hiddenNodeCount += 1;
         node.hidden = true;
@@ -528,9 +523,7 @@ var domFilterer = {
             if ( styleAttr !== '' ) { styleAttr += '; '; }
             node.setAttribute('style', styleAttr + 'display: none !important;');
         }
-        if ( shadowId === undefined ) {
-            return;
-        }
+        if ( shadowId === undefined ) { return; }
         var shadow = node.shadowRoot;
         if ( shadow ) {
             if ( shadow[shadowId] && shadow.firstElementChild !== null ) {
@@ -548,6 +541,10 @@ var domFilterer = {
         }
     },
 
+    init: function() {
+        this.commitTimer = new vAPI.SafeAnimationFrame(this.commit_.bind(this));
+    },
+
     runJob: function(job, fn) {
         switch ( job.t ) {
         case 'has-hide':
@@ -656,10 +653,10 @@ var complexHideNode = function(node) {
     }
 };
 
-/******************************************************************************/
-
 var cssNotHiddenId = ':not([' + domFilterer.hiddenId + '])';
 
+domFilterer.init();
+
 /******************************************************************************/
 
 return domFilterer;
@@ -687,8 +684,6 @@ return domFilterer;
             return;
         }
 
-        vAPI.executionCost.start();
-
         if ( response.noCosmeticFiltering ) {
             vAPI.domFilterer = null;
             vAPI.domSurveyor = null;
@@ -744,12 +739,10 @@ return domFilterer;
         // uBlock's process was fully initialized. When this happens, pages
         // won't be cleaned right after browser launch.
         if ( document.readyState !== 'loading' ) {
-            window.requestAnimationFrame(vAPI.domIsLoaded);
+            (new vAPI.SafeAnimationFrame(vAPI.domIsLoaded)).start();
         } else {
             document.addEventListener('DOMContentLoaded', vAPI.domIsLoaded);
         }
-
-        vAPI.executionCost.stop('domIsLoading/responseHandler');
     };
 
     var url = window.location.href;
@@ -776,13 +769,10 @@ vAPI.domWatcher = (function() {
         addedNodeLists = [],
         addedNodes = [],
         removedNodes = false,
-        safeObserverHandlerTimer = null,
         listeners = [];
 
     var safeObserverHandler = function() {
-        vAPI.executionCost.start();
-
-        safeObserverHandlerTimer = null;
+        safeObserverHandlerTimer.clear();
         var i = addedNodeLists.length,
             nodeList, iNode, node;
         while ( i-- ) {
@@ -808,15 +798,13 @@ vAPI.domWatcher = (function() {
             addedNodes.length = 0;
             removedNodes = false;
         }
-
-        vAPI.executionCost.stop('domWatcher/safeObserverHandler');
     };
 
+    var safeObserverHandlerTimer = new vAPI.SafeAnimationFrame(safeObserverHandler);
+
     // https://github.com/chrisaljoudi/uBlock/issues/205
     // Do not handle added node directly from within mutation observer.
     var observerHandler = function(mutations) {
-        vAPI.executionCost.start();
-
         var nodeList, mutation,
             i = mutations.length;
         while ( i-- ) {
@@ -829,11 +817,9 @@ vAPI.domWatcher = (function() {
                 removedNodes = true;
             }
         }
-        if ( (addedNodeLists.length !== 0 || removedNodes) && safeObserverHandlerTimer === null ) {
-            safeObserverHandlerTimer = window.requestAnimationFrame(safeObserverHandler);
+        if ( addedNodeLists.length !== 0 || removedNodes ) {
+            safeObserverHandlerTimer.start();
         }
-
-        vAPI.executionCost.stop('domWatcher/observerHandler');
     };
 
     var addListener = function(listener) {
@@ -872,9 +858,7 @@ vAPI.domWatcher = (function() {
                 domLayoutObserver.disconnect();
                 domLayoutObserver = null;
             }
-            if ( safeObserverHandlerTimer !== null ) {
-                window.cancelAnimationFrame(safeObserverHandlerTimer);
-            }
+            safeObserverHandlerTimer.clear();
         });
     };
 
@@ -922,7 +906,6 @@ vAPI.domCollapser = (function() {
         if ( requests === null || Array.isArray(requests) === false ) {
             return;
         }
-        vAPI.executionCost.start();
         var selectors = [],
             netSelectorCacheCountMax = response.netSelectorCacheCountMax,
             aa = [ null ],
@@ -973,11 +956,9 @@ vAPI.domCollapser = (function() {
                 }
             );
         }
-        vAPI.executionCost.stop('domCollapser/onProcessed');
     };
 
     var send = function() {
-        vAPI.executionCost.start();
         timer = null;
         // https://github.com/gorhill/uBlock/issues/1927
         // Normalize hostname to avoid trailing dot of FQHN.
@@ -998,7 +979,6 @@ vAPI.domCollapser = (function() {
             }, onProcessed
         );
         roundtripRequests = [];
-        vAPI.executionCost.stop('domCollapser/send');
     };
 
     var process = function(delay) {
@@ -1121,10 +1101,8 @@ vAPI.domCollapser = (function() {
     };
 
     var onResourceFailed = function(ev) {
-        vAPI.executionCost.start();
         vAPI.domCollapser.add(ev.target);
         vAPI.domCollapser.process();
-        vAPI.executionCost.stop('domCollapser/onResourceFailed');
     };
 
     var domChangedHandler = function(nodes) {
@@ -1134,7 +1112,7 @@ vAPI.domCollapser = (function() {
             if ( node.localName === 'iframe' ) {
                 addIFrame(node);
             }
-            if ( node.children.length !== 0 ) {
+            if ( node.children && node.children.length !== 0 ) {
                 var iframes = node.getElementsByTagName('iframe');
                 if ( iframes.length !== 0 ) {
                     addIFrames(iframes);
@@ -1199,13 +1177,12 @@ vAPI.domSurveyor = (function() {
         cosmeticSurveyingMissCount = 0,
         highGenerics = null,
         lowGenericSelectors = [],
-        queriedSelectors = Object.create(null);
+        queriedSelectors = createSet(),
+        surveyCost = 0;
 
     // Handle main process' response.
 
     var surveyPhase3 = function(response) {
-        vAPI.executionCost.start();
-
         var result = response && response.result,
             firstSurvey = highGenerics === null;
 
@@ -1219,6 +1196,7 @@ vAPI.domSurveyor = (function() {
         }
 
         if ( highGenerics ) {
+            var t0 = window.performance.now();
             if ( highGenerics.hideLowCount ) {
                 processHighLowGenerics(highGenerics.hideLow);
             }
@@ -1228,6 +1206,7 @@ vAPI.domSurveyor = (function() {
             if ( highGenerics.hideHighSimpleCount || highGenerics.hideHighComplexCount ) {
                 processHighHighGenerics();
             }
+            surveyCost += window.performance.now() - t0;
         }
 
         // Need to do this before committing DOM filterer, as needed info
@@ -1239,7 +1218,9 @@ vAPI.domSurveyor = (function() {
                     what: 'cosmeticFiltersInjected',
                     type: 'cosmetic',
                     hostname: window.location.hostname,
-                    selectors: domFilterer.job0._0
+                    selectors: domFilterer.job0._0,
+                    first: firstSurvey,
+                    cost: surveyCost
                 }
             );
         }
@@ -1253,8 +1234,6 @@ vAPI.domSurveyor = (function() {
 
         domFilterer.commit(surveyPhase3Nodes);
         surveyPhase3Nodes = [];
-
-        vAPI.executionCost.stop('domSurveyor/surveyPhase3');
     };
 
     // Query main process.
@@ -1447,42 +1426,35 @@ vAPI.domSurveyor = (function() {
     // http://jsperf.com/enumerate-classes/6
 
     var surveyPhase1 = function(addedNodes) {
-        var nodes = selectNodes('[class],[id]', addedNodes);
-        var qq = queriedSelectors;
-        var ll = lowGenericSelectors;
-        var node, v, vv, j;
-        var i = nodes.length;
+        var t0 = window.performance.now(),
+            nodes = selectNodes('[class],[id]', addedNodes),
+            qq = queriedSelectors,
+            ll = lowGenericSelectors,
+            node, v, vv, j,
+            i = nodes.length;
         while ( i-- ) {
             node = nodes[i];
             if ( node.nodeType !== 1 ) { continue; }
             v = node.id;
             if ( v !== '' && typeof v === 'string' ) {
                 v = '#' + v.trim();
-                if ( v !== '#' && qq[v] === undefined ) {
-                    ll.push(v);
-                    qq[v] = true;
-                }
+                if ( v !== '#' && !qq.has(v) ) { ll.push(v); qq.add(v); }
             }
             vv = node.className;
             if ( vv === '' || typeof vv !== 'string' ) { continue; }
             if ( /\s/.test(vv) === false ) {
                 v = '.' + vv;
-                if ( qq[v] === undefined ) {
-                    ll.push(v);
-                    qq[v] = true;
-                }
+                if ( !qq.has(v) ) { ll.push(v); qq.add(v); }
             } else {
                 vv = node.classList;
                 j = vv.length;
                 while ( j-- ) {
                     v = '.' + vv[j];
-                    if ( qq[v] === undefined ) {
-                        ll.push(v);
-                        qq[v] = true;
-                    }
+                    if ( !qq.has(v) ) { ll.push(v); qq.add(v); }
                 }
             }
         }
+        surveyCost += window.performance.now() - t0;
         surveyPhase2(addedNodes);
     };
 
@@ -1532,8 +1504,6 @@ vAPI.domIsLoaded = function(ev) {
     }
     vAPI.domIsLoaded = null;
 
-    vAPI.executionCost.start();
-
     vAPI.domWatcher.start();
     vAPI.domCollapser.start();
 
@@ -1591,12 +1561,8 @@ vAPI.domIsLoaded = function(ev) {
             document.removeEventListener('mousedown', onMouseClick, true);
         });
     })();
-
-    vAPI.executionCost.stop('domIsLoaded');
 };
 
 /******************************************************************************/
 /******************************************************************************/
 /******************************************************************************/
-
-vAPI.executionCost.stop('contentscript.js');
