@@ -218,7 +218,7 @@ var FilterParser = function() {
     this.hostnames = [];
     this.invalid = false;
     this.cosmetic = true;
-    this.reNeedHostname = /^(?:script:contains|script:inject|.+?:has|.+?:matches-css|:xpath)\(.+?\)$/;
+    this.reNeedHostname = /^(?:script:contains|script:inject|.+?:has|.+?:matches-css(?:-before|-after)?|:xpath)\(.+?\)$/;
 };
 
 /******************************************************************************/
@@ -410,10 +410,15 @@ SelectorCacheEntry.prototype.dispose = function() {
 
 /******************************************************************************/
 
-SelectorCacheEntry.prototype.addCosmetic = function(selectors) {
-    var i = selectors.length || 0;
-    if ( i === 0 ) {
-        this.cosmeticSurveyingMissCount += 1;
+SelectorCacheEntry.prototype.addCosmetic = function(details) {
+    var selectors = details.selectors,
+        i = selectors.length || 0;
+    // https://github.com/gorhill/uBlock/issues/2011
+    //   Avoiding seemingly pointless surveys only if they appear costly.
+    if ( details.first && i === 0 ) {
+        if ( (details.cost || 0) >= 80 ) {
+            this.cosmeticSurveyingMissCount += 1;
+        }
         return;
     }
     this.cosmeticSurveyingMissCount = 0;
@@ -474,12 +479,12 @@ SelectorCacheEntry.prototype.addNetMany = function(selectors, now) {
 
 /******************************************************************************/
 
-SelectorCacheEntry.prototype.add = function(selectors, type) {
+SelectorCacheEntry.prototype.add = function(details) {
     this.lastAccessTime = Date.now();
-    if ( type === 'cosmetic' ) {
-        this.addCosmetic(selectors);
+    if ( details.type === 'cosmetic' ) {
+        this.addCosmetic(details);
     } else {
-        this.addNet(selectors);
+        this.addNet(details.selectors);
     }
 };
 
@@ -590,8 +595,10 @@ var FilterContainer = function() {
     this.selectorCacheTimer = null;
     this.reHasUnicode = /[^\x00-\x7F]/;
     this.reClassOrIdSelector = /^[#.][\w-]+$/;
-    this.rePlainSelector = /^[#.][\w-]+/;
+    this.rePlainSelector = /^[#.][\w\\-]+/;
+    this.rePlainSelectorEscaped = /^[#.](?:\\[0-9A-Fa-f]+ |\\.|\w|-)+/;
     this.rePlainSelectorEx = /^[^#.\[(]+([#.][\w-]+)/;
+    this.reEscapeSequence = /\\([0-9A-Fa-f]+ |.)/g;
     this.reHighLow = /^[a-z]*\[(?:alt|title)="[^"]+"\]$/;
     this.reHighMedium = /^\[href\^="https?:\/\/([^"]{8})[^"]*"\]$/;
     this.reScriptSelector = /^script:(contains|inject)\((.+)\)$/;
@@ -714,7 +721,7 @@ FilterContainer.prototype.isValidSelector = (function() {
     }
 
     var reHasSelector = /^(.+?):has\((.+?)\)$/,
-        reMatchesCSSSelector = /^(.+?):matches-css\((.+?)\)$/,
+        reMatchesCSSSelector = /^(.+?):matches-css(?:-before|-after)?\((.+?)\)$/,
         reXpathSelector = /^:xpath\((.+?)\)$/,
         reStyleSelector = /^(.+?):style\((.+?)\)$/,
         reStyleBad = /url\([^)]+\)/,
@@ -781,6 +788,40 @@ FilterContainer.prototype.isValidSelector = (function() {
 
 /******************************************************************************/
 
+// https://github.com/gorhill/uBlock/issues/1668
+//   The key must be literal: unescape escaped CSS before extracting key.
+//   It's an uncommon case, so it's best to unescape only when needed.
+
+FilterContainer.prototype.keyFromSelector = function(selector) {
+    var matches = this.rePlainSelector.exec(selector);
+    if ( matches === null ) { return; }
+    var key = matches[0];
+    if ( key.indexOf('\\') === -1 ) {
+        return key;
+    }
+    key = '';
+    matches = this.rePlainSelectorEscaped.exec(selector);
+    if ( matches === null ) { return; }
+    var escaped = matches[0],
+        beg = 0;
+    this.reEscapeSequence.lastIndex = 0;
+    for (;;) {
+        matches = this.reEscapeSequence.exec(escaped);
+        if ( matches === null ) {
+            return key + escaped.slice(beg);
+        }
+        key += escaped.slice(beg, matches.index);
+        beg = this.reEscapeSequence.lastIndex;
+        if ( matches[1].length === 1 ) {
+            key += matches[1];
+        } else {
+            key += String.fromCharCode(parseInt(matches[1], 16));
+        }
+    }
+};
+
+/******************************************************************************/
+
 FilterContainer.prototype.compile = function(s, out) {
     var parsed = this.parser.parse(s);
     if ( parsed.cosmetic === false ) {
@@ -841,23 +882,21 @@ FilterContainer.prototype.compileGenericSelector = function(parsed, out) {
 FilterContainer.prototype.compileGenericHideSelector = function(parsed, out) {
     var selector = parsed.suffix,
         type = selector.charAt(0),
-        matches;
+        key, matches;
 
     if ( type === '#' || type === '.' ) {
-        matches = this.rePlainSelector.exec(selector);
-        if ( matches === null ) {
-            return;
-        }
+        key = this.keyFromSelector(selector);
+        if ( key === undefined ) { return; }
         // Single-CSS rule: no need to test for whether the selector
         // is valid, the regex took care of this. Most generic selector falls
         // into that category.
-        if ( matches[0] === selector ) {
-            out.push('c\vlg\v' + matches[0]);
+        if ( key === selector ) {
+            out.push('c\vlg\v' + key);
             return;
         }
-        // Many-CSS rules
+        // Composite CSS rule.
         if ( this.isValidSelector(selector) ) {
-            out.push('c\vlg+\v' + matches[0] + '\v' + selector);
+            out.push('c\vlg+\v' + key + '\v' + selector);
         }
         return;
     }
@@ -1237,6 +1276,12 @@ FilterContainer.prototype.createScriptTagFilter = function(hash, hostname, selec
 
 /******************************************************************************/
 
+FilterContainer.prototype.retrieveScriptTagHostnames = function() {
+    return Object.keys(this.scriptTagFilters);
+};
+
+/******************************************************************************/
+
 FilterContainer.prototype.retrieveScriptTagRegex = function(domain, hostname) {
     if ( this.scriptTagFilterCount === 0 ) {
         return;
@@ -1293,46 +1338,37 @@ FilterContainer.prototype.createUserScriptRule = function(hash, hostname, select
 // https://github.com/gorhill/uBlock/issues/1954
 
 // 01234567890123456789
-// script:inject(token)
-//               ^   ^
-//              14   -1
+// script:inject(token[, arg[, ...]])
+//               ^                 ^
+//              14                 -1
 
 FilterContainer.prototype.retrieveUserScripts = function(domain, hostname) {
-    if ( this.userScriptCount === 0 ) {
-        return;
-    }
+    if ( this.userScriptCount === 0 ) { return; }
+    if ( µb.hiddenSettings.ignoreScriptInjectFilters === true ) { return; }
 
     var reng = µb.redirectEngine;
-    if ( !reng ) {
-        return;
-    }
+    if ( !reng ) { return; }
 
     var out = [],
-        scripts = Object.create(null),
+        scripts = new Map(),
         pos = domain.indexOf('.'),
-        entity = pos !== -1 ? domain.slice(0, pos) + '.*' : '',
-        token, content;
+        entity = pos !== -1 ? domain.slice(0, pos) + '.*' : '';
 
     // Implicit
     var hn = hostname;
     for (;;) {
-        token = hn + '.js';
-        if (
-            (scripts[token] === undefined) &&
-            (content = reng.resourceContentFromName(token, 'application/javascript'))
-        ) {
-            scripts[token] = out.length;
-            out.push(content);
-        }
+        this._lookupUserScript(scripts, hn + '.js', reng, out);
         if ( hn === domain ) { break; }
         pos = hn.indexOf('.');
         if ( pos === -1 ) { break; }
         hn = hn.slice(pos + 1);
     }
+    if ( entity !== '' ) {
+        this._lookupUserScript(scripts, entity + '.js', reng, out);
+    }
 
     // Explicit (hash is domain).
-    var selectors = [],
-        selector, bucket;
+    var selectors = [], bucket;
     if ( (bucket = this.userScripts.get(domain)) ) {
         bucket.retrieve(hostname, selectors);
     }
@@ -1341,15 +1377,7 @@ FilterContainer.prototype.retrieveUserScripts = function(domain, hostname) {
     }
     var i = selectors.length;
     while ( i-- ) {
-        selector = selectors[i];
-        token = selector.slice(14, -1);
-        if (
-            (scripts[token] === undefined) &&
-            (content = reng.resourceContentFromName(token, 'application/javascript'))
-        ) {
-            scripts[token] = out.length;
-            out.push(content);
-        }
+        this._lookupUserScript(scripts, selectors[i].slice(14, -1).trim(), reng, out);
     }
 
     if ( out.length === 0 ) {
@@ -1358,7 +1386,7 @@ FilterContainer.prototype.retrieveUserScripts = function(domain, hostname) {
 
     // Exceptions should be rare, so we check for exception only if there are
     // scriptlets returned.
-    var exceptions = [], j;
+    var exceptions = [], j, token;
     if ( (bucket = this.userScripts.get('!' + domain)) ) {
         bucket.retrieve(hostname, exceptions);
     }
@@ -1368,7 +1396,7 @@ FilterContainer.prototype.retrieveUserScripts = function(domain, hostname) {
     i = exceptions.length;
     while ( i-- ) {
         token = exceptions[i].slice(14, -1);
-        if ( (j = scripts[token]) !== undefined ) {
+        if ( (j = scripts.get(token)) !== undefined ) {
             out[j] = '// User script "' + token + '" excepted.\n';
         }
     }
@@ -1376,6 +1404,45 @@ FilterContainer.prototype.retrieveUserScripts = function(domain, hostname) {
     return out.join('\n');
 };
 
+FilterContainer.prototype._lookupUserScript = function(dict, raw, reng, out) {
+    if ( dict.has(raw) ) { return; }
+    var token, args,
+        pos = raw.indexOf(',');
+    if ( pos === -1 ) {
+        token = raw;
+    } else {
+        token = raw.slice(0, pos).trim();
+        args = raw.slice(pos + 1).trim();
+    }
+    var content = reng.resourceContentFromName(token, 'application/javascript');
+    if ( !content ) { return; }
+    if ( args ) {
+        content = this._fillupUserScript(content, args);
+        if ( !content ) { return; }
+    }
+    dict.set(raw, out.length);
+    out.push(content);
+};
+
+// Fill template placeholders. Return falsy if:
+// - At least one argument contains anything else than /\w/ and `.`
+
+FilterContainer.prototype._fillupUserScript = function(content, args) {
+    var i = 1,
+        pos, arg;
+    while ( args !== '' ) {
+        pos = args.indexOf(',');
+        if ( pos === -1 ) { pos = args.length; }
+        arg = args.slice(0, pos).trim().replace(this._reEscapeScriptArg, '\\$&');
+        content = content.replace('{{' + i + '}}', arg);
+        args = args.slice(pos + 1).trim();
+        i++;
+    }
+    return content;
+};
+
+FilterContainer.prototype._reEscapeScriptArg = /[\\'"]/g;
+
 /******************************************************************************/
 
 FilterContainer.prototype.toSelfie = function() {
@@ -1529,7 +1596,7 @@ FilterContainer.prototype.addToSelectorCache = function(details) {
         this.selectorCacheCount += 1;
         this.triggerSelectorCachePruner();
     }
-    entry.add(selectors, details.type);
+    entry.add(details);
 };
 
 /******************************************************************************/
