@@ -98,18 +98,6 @@ var staticFilterTypes = {
     'xhr': 'xmlhttprequest'
 };
 
-var timeOptions = {
-    hour: '2-digit',
-    minute: '2-digit',
-    second: '2-digit',
-    hour12: false
-};
-
-var dateOptions = {
-    month: 'short',
-    day: '2-digit'
-};
-
 /******************************************************************************/
 
 var classNameFromTabId = function(tabId) {
@@ -176,7 +164,7 @@ var filterDecompiler = (function() {
         10: 'other',
         11: 'popunder',
         12: 'document',
-        13: 'elemhide',
+        13: 'generichide',
         14: 'inline-script',
         15: 'popup'
     };
@@ -241,9 +229,14 @@ var filterDecompiler = (function() {
             break;
         case '||a':
         case '||ah':
+            filter += '||' + tfield0;
+            break;
         case '||_':
         case '||_h':
             filter += '||' + tfield0;
+            if ( tfields[1] === '1' ) { // left-anchored?
+                filter += '|';
+            }
             break;
         case '//':
         case '//h':
@@ -263,12 +256,12 @@ var filterDecompiler = (function() {
         case '|ah':
         case 'a|h':
         case '||ah':
-        case '||_h':
         case '//h':
             opts.push('domain=' + tfields[1]);
             break;
         case 'ah':
         case '_h':
+        case '||_h':
             opts.push('domain=' + tfields[2]);
             break;
         default:
@@ -429,6 +422,12 @@ var createHiddenTextNode = function(text) {
 
 /******************************************************************************/
 
+var padTo2 = function(v) {
+    return v < 10 ? '0' + v : v;
+};
+
+/******************************************************************************/
+
 var createGap = function(tabId, url) {
     var tr = createRow('1');
     tr.classList.add('tab');
@@ -541,9 +540,11 @@ var renderLogEntry = function(entry) {
     }
 
     // Fields common to all rows.
-    var time = new Date(entry.tstamp);
-    tr.cells[0].textContent = time.toLocaleTimeString('fullwide', timeOptions);
-    tr.cells[0].title = time.toLocaleDateString('fullwide', dateOptions);
+    var time = logDate;
+    time.setTime(entry.tstamp - logDateTimezoneOffset);
+    tr.cells[0].textContent = padTo2(time.getUTCHours()) + ':' +
+                              padTo2(time.getMinutes()) + ':' +
+                              padTo2(time.getSeconds());
 
     if ( entry.tab ) {
         tr.classList.add('tab');
@@ -561,6 +562,10 @@ var renderLogEntry = function(entry) {
     return tr;
 };
 
+// Reuse date objects.
+var logDate = new Date(),
+    logDateTimezoneOffset = logDate.getTimezoneOffset() * 60000;
+
 /******************************************************************************/
 
 var renderLogEntries = function(response) {
