@@ -199,22 +199,27 @@ var µb = µBlock;
 /******************************************************************************/
 
 var getHostnameDict = function(hostnameToCountMap) {
-    var r = {}, de;
-    var domainFromHostname = µb.URI.domainFromHostname;
-    var domain, counts, blockCount, allowCount;
-    for ( var hostname in hostnameToCountMap ) {
-        if ( hostnameToCountMap.hasOwnProperty(hostname) === false ) {
-            continue;
+    var r = Object.create(null),
+        domainEntry,
+        domainFromHostname = µb.URI.domainFromHostname,
+        domain, blockCount, allowCount,
+        iter = hostnameToCountMap.entries(),
+        entry, hostname, counts;
+    for (;;) {
+        entry = iter.next();
+        if ( entry.done ) {
+            break;
         }
-        if ( r.hasOwnProperty(hostname) ) {
+        hostname = entry.value[0];
+        if ( r[hostname] !== undefined ) {
             continue;
         }
         domain = domainFromHostname(hostname) || hostname;
-        counts = hostnameToCountMap[domain] || 0;
+        counts = hostnameToCountMap.get(domain) || 0;
         blockCount = counts & 0xFFFF;
         allowCount = counts >>> 16 & 0xFFFF;
-        if ( r.hasOwnProperty(domain) === false ) {
-            de = r[domain] = {
+        if ( r[domain] === undefined ) {
+            domainEntry = r[domain] = {
                 domain: domain,
                 blockCount: blockCount,
                 allowCount: allowCount,
@@ -222,13 +227,13 @@ var getHostnameDict = function(hostnameToCountMap) {
                 totalAllowCount: allowCount
             };
         } else {
-            de = r[domain];
+            domainEntry = r[domain];
         }
-        counts = hostnameToCountMap[hostname] || 0;
+        counts = entry.value[1];
         blockCount = counts & 0xFFFF;
         allowCount = counts >>> 16 & 0xFFFF;
-        de.totalBlockCount += blockCount;
-        de.totalAllowCount += allowCount;
+        domainEntry.totalBlockCount += blockCount;
+        domainEntry.totalAllowCount += allowCount;
         if ( hostname === domain ) {
             continue;
         }
@@ -268,10 +273,8 @@ var getFirewallRules = function(srcHostname, desHostnames) {
     r['. * 3p-frame'] = df.evaluateCellZY(srcHostname, '*', '3p-frame').toFilterString();
 
     for ( var desHostname in desHostnames ) {
-        if ( desHostnames.hasOwnProperty(desHostname) ) {
-            r['/ ' + desHostname + ' *'] = df.evaluateCellZY('*', desHostname, '*').toFilterString();
-            r['. ' + desHostname + ' *'] = df.evaluateCellZY(srcHostname, desHostname, '*').toFilterString();
-        }
+        r['/ ' + desHostname + ' *'] = df.evaluateCellZY('*', desHostname, '*').toFilterString();
+        r['. ' + desHostname + ' *'] = df.evaluateCellZY(srcHostname, desHostname, '*').toFilterString();
     }
     return r;
 };
@@ -279,8 +282,8 @@ var getFirewallRules = function(srcHostname, desHostnames) {
 /******************************************************************************/
 
 var popupDataFromTabId = function(tabId, tabTitle) {
-    var tabContext = µb.tabContextManager.mustLookup(tabId);
-    var rootHostname = tabContext.rootHostname;
+    var tabContext = µb.tabContextManager.mustLookup(tabId),
+        rootHostname = tabContext.rootHostname;
     var r = {
         advancedUserEnabled: µb.userSettings.advancedUserEnabled,
         appName: vAPI.app.name,
@@ -291,6 +294,7 @@ var popupDataFromTabId = function(tabId, tabTitle) {
         firewallPaneMinimized: µb.userSettings.firewallPaneMinimized,
         globalAllowedRequestCount: µb.localSettings.allowedRequestCount,
         globalBlockedRequestCount: µb.localSettings.blockedRequestCount,
+        fontSize: µb.hiddenSettings.popupFontSize,
         netFilteringSwitch: false,
         rawURL: tabContext.rawURL,
         pageURL: tabContext.normalURL,
@@ -306,6 +310,17 @@ var popupDataFromTabId = function(tabId, tabTitle) {
 
     var pageStore = µb.pageStoreFromTabId(tabId);
     if ( pageStore ) {
+        // https://github.com/gorhill/uBlock/issues/2105
+        //   Be sure to always include the current page's hostname -- it might
+        //   not be present when the page itself is pulled from the browser's
+        //   short-term memory cache. This needs to be done before calling
+        //   getHostnameDict().
+        if (
+            pageStore.hostnameToCountMap.has(rootHostname) === false &&
+            µb.URI.isNetworkURI(tabContext.rawURL)
+        ) {
+            pageStore.hostnameToCountMap.set(rootHostname, 0);
+        }
         r.pageBlockedRequestCount = pageStore.perLoadBlockedRequestCount;
         r.pageAllowedRequestCount = pageStore.perLoadAllowedRequestCount;
         r.netFilteringSwitch = pageStore.getNetFilteringSwitch();
@@ -466,22 +481,21 @@ var filterRequests = function(pageStore, details) {
 
     //console.debug('messaging.js/contentscript-end.js: processing %d requests', requests.length);
 
-    var hostnameFromURI = µb.URI.hostnameFromURI;
-    var redirectEngine = µb.redirectEngine;
-    var punycodeURL = vAPI.punycodeURL;
+    var hostnameFromURI = µb.URI.hostnameFromURI,
+        redirectEngine = µb.redirectEngine,
+        punycodeURL = vAPI.punycodeURL;
 
     // Create evaluation context
-    var context = pageStore.createContextFromFrameHostname(details.pageHostname);
-
-    var request, r;
-    var i = requests.length;
+    var context = pageStore.createContextFromFrameHostname(details.pageHostname),
+        request, r,
+        i = requests.length;
     while ( i-- ) {
         request = requests[i];
         context.requestURL = punycodeURL(request.url);
         context.requestHostname = hostnameFromURI(context.requestURL);
         context.requestType = tagNameToRequestTypeMap[request.tag];
         r = pageStore.filterRequest(context);
-        if ( typeof r !== 'string' || r.charAt(1) !== 'b' ) {
+        if ( typeof r !== 'string' || r.charCodeAt(1) !== 98 /* 'b' */ ) {
             continue;
         }
         // Redirected? (We do not hide redirected resources.)
@@ -645,16 +659,18 @@ vAPI.messaging.listen('elementPicker', onMessage);
 
 /******************************************************************************/
 
-var µb = µBlock;
-
-/******************************************************************************/
-
 var onMessage = function(request, sender, callback) {
+    // Cloud storage support is optional.
+    if ( µBlock.cloudStorageSupported !== true ) {
+        callback();
+        return;
+    }
+
     // Async
     switch ( request.what ) {
     case 'cloudGetOptions':
         vAPI.cloud.getOptions(function(options) {
-            options.enabled = µb.userSettings.cloudStorageEnabled === true;
+            options.enabled = µBlock.userSettings.cloudStorageEnabled === true;
             callback(options);
         });
         return;
@@ -719,7 +735,9 @@ var getLocalData = function(callback) {
             lastRestoreFile: o.lastRestoreFile,
             lastRestoreTime: o.lastRestoreTime,
             lastBackupFile: o.lastBackupFile,
-            lastBackupTime: o.lastBackupTime
+            lastBackupTime: o.lastBackupTime,
+            cloudStorageSupported: µb.cloudStorageSupported,
+            privacySettingsSupported: µb.privacySettingsSupported
         });
     };
 
@@ -732,6 +750,7 @@ var backupUserData = function(callback) {
         version: vAPI.app.version,
         userSettings: µb.userSettings,
         filterLists: {},
+        hiddenSettingsString: µb.stringFromHiddenSettings(),
         netWhitelist: µb.stringFromWhitelist(µb.netWhitelist),
         dynamicFilteringString: µb.permanentFirewall.toString(),
         urlFilteringString: µb.permanentURLFiltering.toString(),
@@ -742,9 +761,8 @@ var backupUserData = function(callback) {
     var onSelectedListsReady = function(filterLists) {
         userData.filterLists = filterLists;
 
-        var now = new Date();
         var filename = vAPI.i18n('aboutBackupFilename')
-            .replace('{{datetime}}', now.toLocaleString())
+            .replace('{{datetime}}', µb.dateNowToSensibleString())
             .replace(/ +/g, '_');
 
         vAPI.download({
@@ -783,13 +801,9 @@ var restoreUserData = function(request) {
         µBlock.saveLocalSettings();
         vAPI.storage.set(userData.userSettings, onCountdown);
         µb.keyvalSetOne('remoteBlacklists', userData.filterLists, onCountdown);
+        µb.hiddenSettingsFromString(userData.hiddenSettingsString || '');
         µb.keyvalSetOne('netWhitelist', userData.netWhitelist || '', onCountdown);
-
-        // With versions 0.9.2.4-, dynamic rules were saved within the
-        // `userSettings` object. No longer the case.
-        var s = userData.dynamicFilteringString || userData.userSettings.dynamicFilteringString || '';
-        µb.keyvalSetOne('dynamicFilteringString', s, onCountdown);
-
+        µb.keyvalSetOne('dynamicFilteringString', userData.dynamicFilteringString || '', onCountdown);
         µb.keyvalSetOne('urlFilteringString', userData.urlFilteringString || '', onCountdown);
         µb.keyvalSetOne('hostnameSwitchesString', userData.hostnameSwitchesString || '', onCountdown);
         µb.assets.put(µb.userFiltersPath, userData.userFilters, onCountdown);
@@ -807,11 +821,14 @@ var restoreUserData = function(request) {
 
     // If we are going to restore all, might as well wipe out clean local
     // storage
+    vAPI.cacheStorage.clear();
     vAPI.storage.clear(onAllRemoved);
 };
 
 var resetUserData = function() {
+    vAPI.cacheStorage.clear();
     vAPI.storage.clear();
+    vAPI.localStorage.removeItem('hiddenSettings');
 
     // Keep global counts, people can become quite attached to numbers
     µb.saveLocalSettings();
@@ -956,6 +973,10 @@ var onMessage = function(request, sender, callback) {
         µb.assets.purgeCacheableAsset(request.path);
         break;
 
+    case 'readHiddenSettings':
+        response = µb.stringFromHiddenSettings();
+        break;
+
     case 'restoreUserData':
         restoreUserData(request);
         break;
@@ -986,6 +1007,10 @@ var onMessage = function(request, sender, callback) {
         response = getRules();
         break;
 
+    case 'writeHiddenSettings':
+        µb.hiddenSettingsFromString(request.content);
+        break;
+
     default:
         return vAPI.messaging.UNHANDLED;
     }
