@@ -19,7 +19,7 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* global publicSuffixList */
+/* global objectAssign, publicSuffixList */
 
 'use strict';
 
@@ -61,24 +61,28 @@ var onAllReady = function() {
     µb.assetUpdater.onStart.addEventListener(µb.updateStartHandler.bind(µb));
     µb.assetUpdater.onCompleted.addEventListener(µb.updateCompleteHandler.bind(µb));
     µb.assetUpdater.onAssetUpdated.addEventListener(µb.assetUpdatedHandler.bind(µb));
-    µb.assets.onAssetCacheRemoved.addEventListener(µb.assetCacheRemovedHandler.bind(µb));
+    µb.assets.onAssetRemoved.addListener(µb.assetCacheRemovedHandler.bind(µb));
 
     // Important: remove barrier to remote fetching, this was useful only
     // for launch time.
     µb.assets.remoteFetchBarrier -= 1;
 
-    vAPI.cloud.start([
-        'tpFiltersPane',
-        'myFiltersPane',
-        'myRulesPane',
-        'whitelistPane'
-    ]);
+    // vAPI.cloud is optional.
+    if ( µb.cloudStorageSupported ) {
+        vAPI.cloud.start([
+            'tpFiltersPane',
+            'myFiltersPane',
+            'myRulesPane',
+            'whitelistPane'
+        ]);
+    }
 
     //quickProfiler.stop(0);
 
-    vAPI.onLoadAllCompleted();
     µb.contextMenu.update(null);
     µb.firstInstall = false;
+
+    vAPI.net.onReady();
 };
 
 /******************************************************************************/
@@ -95,24 +99,17 @@ var onPSLReady = function() {
 // To bring older versions up to date
 
 var onVersionReady = function(lastVersion) {
-    // Whitelist some key scopes by default
-    if ( lastVersion.localeCompare('0.8.6.0') < 0 ) {
-        µb.netWhitelist = µb.whitelistFromString(
-            µb.stringFromWhitelist(µb.netWhitelist) +
-            '\n' +
-            µb.netWhitelistDefault
-        );
-        µb.saveWhitelist();
-    }
-    // https://github.com/gorhill/uBlock/issues/135#issuecomment-96677379
-    // `about:loopconversation` is used by Firefox for its Hello service
-    if ( lastVersion.localeCompare('0.9.5.2') < 0 ) {
-        µb.netWhitelist = µb.whitelistFromString(
-            µb.stringFromWhitelist(µb.netWhitelist) +
-            '\n' +
-            'loopconversation.about-scheme'
-        );
-        µb.saveWhitelist();
+    // Starting with 1.9.17, non-advanced users can have access to the dynamic
+    // filtering pane in read-only mode. Still, it should not be visible by
+    // default.
+    if ( lastVersion.localeCompare('1.9.17') < 0 ) {
+        if (
+            µb.userSettings.advancedUserEnabled === false &&
+            µb.userSettings.dynamicFilteringEnabled === true
+        ) {
+            µb.userSettings.dynamicFilteringEnabled = false;
+            µb.keyvalSetOne('dynamicFilteringEnabled', false);
+        }
     }
     if ( lastVersion !== vAPI.app.version ) {
         vAPI.storage.set({ version: vAPI.app.version });
@@ -166,11 +163,13 @@ var onUserSettingsReady = function(fetched) {
     µb.assets.autoUpdate = userSettings.autoUpdate;
     µb.assets.autoUpdateDelay = µb.updateAssetsEvery;
 
-    vAPI.browserSettings.set({
-        'hyperlinkAuditing': !userSettings.hyperlinkAuditingDisabled,
-        'prefetching': !userSettings.prefetchingDisabled,
-        'webrtcIPAddress': !userSettings.webrtcIPAddressHidden
-    });
+    if ( µb.privacySettingsSupported ) {
+        vAPI.browserSettings.set({
+            'hyperlinkAuditing': !userSettings.hyperlinkAuditingDisabled,
+            'prefetching': !userSettings.prefetchingDisabled,
+            'webrtcIPAddress': !userSettings.webrtcIPAddressHidden
+        });
+    }
 
     µb.permanentFirewall.fromString(fetched.dynamicFilteringString);
     µb.sessionFirewall.assign(µb.permanentFirewall);
@@ -184,10 +183,6 @@ var onUserSettingsReady = function(fetched) {
     if ( µb.firstInstall && vAPI.battery ) {
         userSettings.ignoreGenericCosmeticFilters = true;
     }
-
-    // Remove obsolete setting
-    delete userSettings.logRequests;
-    vAPI.storage.remove('logRequests');
 };
 
 /******************************************************************************/
@@ -271,7 +266,7 @@ var onAdminSettingsRestored = function() {
         'lastRestoreTime': 0,
         'lastBackupFile': '',
         'lastBackupTime': 0,
-        'netWhitelist': '',
+        'netWhitelist': µb.netWhitelistDefault,
         'selfie': null,
         'selfieMagic': '',
         'version': '0.0.0.0'
@@ -286,6 +281,28 @@ var onAdminSettingsRestored = function() {
 
 /******************************************************************************/
 
+µb.hiddenSettings = (function() {
+    var out = objectAssign({}, µb.hiddenSettingsDefault),
+        json = vAPI.localStorage.getItem('hiddenSettings');
+    if ( typeof json === 'string' ) {
+        try {
+            var o = JSON.parse(json);
+            if ( o instanceof Object ) {
+                for ( var k in o ) {
+                    if ( out.hasOwnProperty(k) ) {
+                        out[k] = o[k];
+                    }
+                }
+            }
+        }
+        catch(ex) {
+        }
+    }
+    return out;
+})();
+
+/******************************************************************************/
+
 return function() {
     // https://github.com/gorhill/uBlock/issues/531
     µb.restoreAdminSettings(onAdminSettingsRestored);
