@@ -68,7 +68,7 @@ var typeNameToTypeValue = {
              'other': 10 << 4,
           'popunder': 11 << 4,
         'main_frame': 12 << 4,
-          'elemhide': 13 << 4,
+       'generichide': 13 << 4,
      'inline-script': 14 << 4,
              'popup': 15 << 4
 };
@@ -87,7 +87,7 @@ var typeValueToTypeName = {
     10: 'other',
     11: 'popunder',
     12: 'document',
-    13: 'elemhide',
+    13: 'generichide',
     14: 'inline-script',
     15: 'popup'
 };
@@ -106,6 +106,9 @@ var AllowAnyTypeAnyParty = AllowAction | AnyType | AnyParty;
 var AllowAnyType = AllowAction | AnyType;
 var AllowAnyParty = AllowAction | AnyParty;
 
+var genericHideException = AllowAction | AnyParty | typeNameToTypeValue.generichide,
+    genericHideImportant = BlockAction | AnyParty | typeNameToTypeValue.generichide | Important;
+
 // ABP filters: https://adblockplus.org/en/filters
 // regex tester: http://regex101.com/
 
@@ -337,57 +340,42 @@ var hostnameMissTest = function(owner) {
 };
 
 var hostnameHitSetTest = function(owner) {
-    var dict = owner._hostnameDict;
-    var needle = pageHostnameRegister;
-    var pos;
+    var dict = owner._hostnameDict,
+        needle = pageHostnameRegister,
+        pos;
     for (;;) {
-        if ( dict.has(needle) ) {
-            return true;
-        }
+        if ( dict.has(needle) ) { return true; }
         pos = needle.indexOf('.');
-        if ( pos === -1 ) {
-            break;
-        }
+        if ( pos === -1 ) { break; }
         needle = needle.slice(pos + 1);
     }
     return false;
 };
 
 var hostnameMissSetTest = function(owner) {
-    var dict = owner._hostnameDict;
-    var needle = pageHostnameRegister;
-    var pos;
+    var dict = owner._hostnameDict,
+        needle = pageHostnameRegister,
+        pos;
     for (;;) {
-        if ( dict.has(needle) ) {
-            return false;
-        }
+        if ( dict.has(needle) ) { return false; }
         pos = needle.indexOf('.');
-        if ( pos === -1 ) {
-            break;
-        }
+        if ( pos === -1 ) { break; }
         needle = needle.slice(pos + 1);
     }
-
     return true;
 };
 
 var hostnameMixedSetTest = function(owner) {
-    var dict = owner._hostnameDict;
-    var needle = pageHostnameRegister;
-    var hit = false;
-    var v, pos;
+    var dict = owner._hostnameDict,
+        needle = pageHostnameRegister,
+        hit = false,
+        v, pos;
     for (;;) {
         v = dict.get(needle);
-        if ( v === false ) {
-            return false;
-        }
-        if ( v === true ) {
-            hit = true;
-        }
+        if ( v === false ) { return false; }
+        if ( v === true ) { hit = true; }
         pos = needle.indexOf('.');
-        if ( pos === -1 ) {
-            break;
-        }
+        if ( pos === -1 ) { break; }
         needle = needle.slice(pos + 1);
     }
     return hit;
@@ -858,14 +846,15 @@ FilterGenericHostname.fromSelfie = function(s) {
 // whether the start of the match falls within the hostname part of the
 // URL.
 
-var FilterGenericHnAnchored = function(s) {
+var FilterGenericHnAnchored = function(s, anchor) {
     this.s = s;
+    this.anchor = anchor;
     this.re = null;
 };
 
 FilterGenericHnAnchored.prototype.match = function(url) {
     if ( this.re === null ) {
-        this.re = strToRegex('||' + this.s, 0);
+        this.re = strToRegex('||' + this.s, this.anchor);
     }
     var matchStart = url.search(this.re);
     return matchStart !== -1 && isHnAnchored(url, matchStart);
@@ -877,21 +866,22 @@ FilterGenericHnAnchored.prototype.rtfid = '||_';
 
 FilterGenericHnAnchored.prototype.toSelfie =
 FilterGenericHnAnchored.prototype.rtCompile = function() {
-    return this.s;
+    return this.s + '\t' + this.anchor;
 };
 
 FilterGenericHnAnchored.compile = function(details) {
-    return details.f;
+    return details.f + '\t' + details.anchor;
 };
 
 FilterGenericHnAnchored.fromSelfie = function(s) {
-    return new FilterGenericHnAnchored(s);
+    var pos = s.indexOf('\t');
+    return new FilterGenericHnAnchored(s.slice(0, pos), parseInt(s.slice(pos + 1), 10));
 };
 
 /******************************************************************************/
 
-var FilterGenericHnAnchoredHostname = function(s, domainOpt) {
-    FilterGenericHnAnchored.call(this, s);
+var FilterGenericHnAnchoredHostname = function(s, anchor, domainOpt) {
+    FilterGenericHnAnchored.call(this, s, anchor);
     this.domainOpt = domainOpt;
     this.hostnameTest = hostnameTestPicker(this);
 };
@@ -909,16 +899,16 @@ FilterGenericHnAnchoredHostname.prototype.rtfid = '||_h';
 
 FilterGenericHnAnchoredHostname.prototype.toSelfie =
 FilterGenericHnAnchoredHostname.prototype.rtCompile = function() {
-    return this.s + '\t' + this.domainOpt;
+    return this.s + '\t' + this.anchor + '\t' + this.domainOpt;
 };
 
 FilterGenericHnAnchoredHostname.compile = function(details) {
-    return details.f + '\t' + details.domainOpt;
+    return details.f + '\t' + details.anchor + '\t' + details.domainOpt;
 };
 
 FilterGenericHnAnchoredHostname.fromSelfie = function(s) {
-    var pos = s.indexOf('\t');
-    return new FilterGenericHnAnchoredHostname(s.slice(0, pos), s.slice(pos + 1));
+    var fields = s.split('\t');
+    return new FilterGenericHnAnchoredHostname(fields[0], parseInt(fields[1], 10), fields[2]);
 };
 
 /******************************************************************************/
@@ -1164,6 +1154,9 @@ var FilterParser = function() {
 
 /******************************************************************************/
 
+// https://github.com/gorhill/uBlock/issues/1493
+//   Transpose `ping` into `other` for now.
+
 FilterParser.prototype.toNormalizedType = {
         'stylesheet': 'stylesheet',
              'image': 'image',
@@ -1176,9 +1169,12 @@ FilterParser.prototype.toNormalizedType = {
              'media': 'media',
          'websocket': 'websocket',
              'other': 'other',
+            'beacon': 'other',
+              'ping': 'other',
           'popunder': 'popunder',
           'document': 'main_frame',
-          'elemhide': 'elemhide',
+       'generichide': 'generichide',
+          'elemhide': 'generichide',
      'inline-script': 'inline-script',
              'popup': 'popup'
 };
@@ -1269,8 +1265,8 @@ FilterParser.prototype.parseOptions = function(s) {
         // `generichide` concept already supported, just a matter of
         // adding support for the new keyword.
         if ( opt === 'elemhide' || opt === 'generichide' ) {
-            if ( this.action === AllowAction ) {
-                this.parseOptType('elemhide', false);
+            if ( not === false ) {
+                this.parseOptType('generichide', false);
                 continue;
             }
             this.unsupported = true;
@@ -1739,7 +1735,7 @@ FilterContainer.prototype.getFilterClass = function(details) {
         if ( details.isRegex ) {
             return FilterRegexHostname;
         }
-        if ( this.reIsGeneric.test(s) ) {
+        if ( this.reIsGeneric.test(s) || details.token === '*' ) {
             if ( details.hostnameAnchored ) {
                 return FilterGenericHnAnchoredHostname;
             }
@@ -1749,6 +1745,10 @@ FilterContainer.prototype.getFilterClass = function(details) {
             return FilterPlainLeftAnchoredHostname;
         }
         if ( details.anchor > 0 ) {
+            // https://github.com/gorhill/uBlock/issues/1669
+            if ( details.hostnameAnchored ) {
+                return FilterGenericHnAnchoredHostname;
+            }
             return FilterPlainRightAnchoredHostname;
         }
         if ( details.hostnameAnchored ) {
@@ -1766,7 +1766,7 @@ FilterContainer.prototype.getFilterClass = function(details) {
     if ( details.isRegex ) {
         return FilterRegex;
     }
-    if ( this.reIsGeneric.test(s) ) {
+    if ( this.reIsGeneric.test(s) || details.token === '*' ) {
         if ( details.hostnameAnchored ) {
             return FilterGenericHnAnchored;
         }
@@ -1776,6 +1776,10 @@ FilterContainer.prototype.getFilterClass = function(details) {
         return FilterPlainLeftAnchored;
     }
     if ( details.anchor > 0 ) {
+        // https://github.com/gorhill/uBlock/issues/1669
+        if ( details.hostnameAnchored ) {
+            return FilterGenericHnAnchored;
+        }
         return FilterPlainRightAnchored;
     }
     if ( details.hostnameAnchored ) {
@@ -2163,9 +2167,7 @@ FilterContainer.prototype.matchTokens = function(bucket, url) {
     for (;;) {
         tokenEntry = tokens[i++];
         token = tokenEntry.token;
-        if ( token === '' ) {
-            break;
-        }
+        if ( !token ) { break; }
         f = bucket.get(token);
         if ( f !== undefined && f.match(url, tokenEntry.beg) ) {
             this.tokenRegister = token;
@@ -2189,11 +2191,47 @@ FilterContainer.prototype.matchTokens = function(bucket, url) {
 
 // Specialized handlers
 
+// https://github.com/gorhill/uBlock/issues/1477
+//   Special case: blocking-generichide filter ALWAYS exists, it is implicit --
+//   thus we always first check for exception filters, then for important block
+//   filter if and only if there was a hit on an exception filter.
+// https://github.com/gorhill/uBlock/issues/2103
+//   User may want to override `generichide` exception filters.
+
+FilterContainer.prototype.matchStringGenericHide = function(context, requestURL) {
+    var url = this.urlTokenizer.setURL(requestURL);
+
+    // https://github.com/gorhill/uBlock/issues/2225
+    //   Important: this is used by FilterHostnameDict.match().
+    requestHostnameRegister = µb.URI.hostnameFromURI(url);
+
+    var bucket = this.categories.get(toHex(genericHideException));
+    if ( !bucket || this.matchTokens(bucket, url) === false ) {
+        this.fRegister = null;
+        return;
+    }
+
+    bucket = this.categories.get(toHex(genericHideImportant));
+    if ( bucket && this.matchTokens(bucket, url) ) {
+        this.keyRegister = genericHideImportant;
+        return true;
+    }
+
+    this.keyRegister = genericHideException;
+    return false;
+};
+
+/******************************************************************************/
+
 // https://github.com/chrisaljoudi/uBlock/issues/116
-// Some type of requests are exceptional, they need custom handling,
-// not the generic handling.
+//   Some type of requests are exceptional, they need custom handling,
+//   not the generic handling.
 
 FilterContainer.prototype.matchStringExactType = function(context, requestURL, requestType) {
+    // Special case.
+    if ( requestType === 'generichide' ) {
+        return this.matchStringGenericHide(context, requestURL);
+    }
     // Be prepared to support unknown types
     var type = typeNameToTypeValue[requestType];
     if ( type === undefined ) {
@@ -2207,30 +2245,14 @@ FilterContainer.prototype.matchStringExactType = function(context, requestURL, r
     pageHostnameRegister = context.pageHostname || '';
     requestHostnameRegister = µb.URI.hostnameFromURI(url);
 
-    var party = isFirstParty(context.pageDomain, requestHostnameRegister) ? FirstParty : ThirdParty;
+    var party = isFirstParty(context.pageDomain, requestHostnameRegister) ? FirstParty : ThirdParty,
+        categories = this.categories,
+        key, bucket;
 
     this.fRegister = null;
 
-    var categories = this.categories;
-    var key, bucket;
-
-    // https://github.com/gorhill/uBlock/issues/1477
-    // Special case: blocking elemhide filter ALWAYS exists, it is implicit --
-    // thus we always and only check for exception filters.
-    if ( requestType === 'elemhide' ) {
-        key = AllowAnyParty | type;
-        if (
-            (bucket = categories.get(toHex(key))) &&
-            this.matchTokens(bucket, url)
-        ) {
-            this.keyRegister = key;
-            return false;
-        }
-        return undefined;
-    }
-
     // https://github.com/chrisaljoudi/uBlock/issues/139
-    // Test against important block filters
+    //   Test against important block filters
     key = BlockAnyParty | Important | type;
     if ( (bucket = categories.get(toHex(key))) ) {
         if ( this.matchTokens(bucket, url) ) {
@@ -2264,7 +2286,7 @@ FilterContainer.prototype.matchStringExactType = function(context, requestURL, r
 
     // If there is no block filter, no need to test against allow filters
     if ( this.fRegister === null ) {
-        return undefined;
+        return;
     }
 
     // Test against allow filters
@@ -2401,7 +2423,7 @@ FilterContainer.prototype.matchString = function(context) {
 
     // If there is no block filter, no need to test against allow filters
     if ( this.fRegister === null ) {
-        return undefined;
+        return;
     }
 
     // Test against allow filters
