@@ -21,6 +21,8 @@
 
 /* global publicSuffixList */
 
+'use strict';
+
 /*******************************************************************************
 
 RFC 3986 as reference: http://tools.ietf.org/html/rfc3986#appendix-A
@@ -33,8 +35,6 @@ Naming convention from https://en.wikipedia.org/wiki/URI_scheme#Examples
 
 µBlock.URI = (function() {
 
-'use strict';
-
 /******************************************************************************/
 
 // Favorite regex tool: http://regex101.com/
@@ -402,6 +402,18 @@ URI.domainFromURI = function(uri) {
 
 /******************************************************************************/
 
+URI.isNetworkURI = function(uri) {
+    return /^(?:ftps?|https?|wss?):\/\//.test(uri);
+};
+
+/******************************************************************************/
+
+URI.isNetworkScheme = function(scheme) {
+    return /^(?:ftps?|https?|wss?)$/.test(scheme);
+};
+
+/******************************************************************************/
+
 // Normalize the way µBlock expects it
 
 URI.normalizedURI = function() {
