@@ -9,26 +9,26 @@ DES=dist/build/uBlock0.chromium
 rm -rf $DES
 mkdir -p $DES
 
-cp -R assets $DES/
-rm $DES/assets/*.sh
-cp -R src/css $DES/
-cp -R src/img $DES/
-cp -R src/js $DES/
-cp -R src/lib $DES/
-cp -R src/_locales $DES/
-cp -R $DES/_locales/nb $DES/_locales/no
-cp src/*.html $DES/
-cp platform/chromium/*.js $DES/js/
+bash ./tools/make-assets.sh $DES
+
+cp -R src/css               $DES/
+cp -R src/img               $DES/
+cp -R src/js                $DES/
+cp -R src/lib               $DES/
+cp -R src/_locales          $DES/
+cp -R $DES/_locales/nb      $DES/_locales/no
+cp src/*.html               $DES/
+cp platform/chromium/*.js   $DES/js/
 cp -R platform/chromium/img $DES/
 cp platform/chromium/*.html $DES/
 cp platform/chromium/*.json $DES/
-cp LICENSE.txt $DES/
+cp LICENSE.txt              $DES/
 
 if [ "$1" = all ]; then
     echo "*** uBlock0.chromium: Creating package..."
-    pushd $(dirname $DES/)
+    pushd $(dirname $DES/) > /dev/null
     zip uBlock0.chromium.zip -qr $(basename $DES/)/*
-    popd
+    popd > /dev/null
 fi
 
 echo "*** uBlock0.chromium: Package done."
